package cn.fw.valhalla.common.utils;

import cn.fw.common.exception.BusinessException;
import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author xiatian
 * @since 2022-06-06
 */
@Slf4j
public class BeanUtils extends org.springframework.beans.BeanUtils {

    /**
     * 对象拷贝；使用注意核对字段
     *
     * @param o
     * @param clazz
     * @param <O>
     * @param <T>
     * @return
     */
    public static <O, T> T copy(O o, Class<T> clazz) {
        try {
            T t = clazz.newInstance();
            copyProperties(o, t);
            return t;
        } catch (Exception e) {
            log.error("对象拷贝错误！", e);
            throw new BusinessException("对象拷贝错误！");
        }
    }

    /**
     * 对象列表拷贝
     *
     * @param originList
     * @param targetClass
     * @param <O>
     * @param <T>
     * @return
     */
    public static <O, T> List<T> copyList(Collection<O> originList, Class<T> targetClass) {
        if (CollectionUtils.isEmpty(originList)) {
            return Collections.emptyList();
        }

        return originList.stream().map(o -> copy(o, targetClass)).collect(Collectors.toList());
    }

    /**
     * 把对象转为map
     *
     * @param bean
     * @param <K>
     * @param <V>
     * @return
     */
    public static <K, V> Map<K, V> toMap(Object bean) {
        ObjectMapper m = new ObjectMapper();
        //noinspection unchecked
        return m.convertValue(bean, Map.class);
    }

    public static Map<String, String> toStringMap(Object bean) {
        ObjectMapper m = new ObjectMapper();
        String jsonStr;
        try {
            jsonStr = m.writeValueAsString(bean);
        } catch (JsonProcessingException e) {
            log.error("【json序列化失败】", e);
            throw new BusinessException("系统异常");
        }

        Map<String, String> stringMap = Maps.newHashMap();
        @SuppressWarnings("unchecked")
        Map<String, Object> map = (Map<String, Object>) JSON.parse(jsonStr);
        map.keySet().stream().filter(Objects::nonNull)
                .forEach(k -> Optional.ofNullable(map.get(k))
                        .ifPresent(v -> stringMap.put(k, String.valueOf(v))));
        return stringMap;
    }
}
