package cn.fw.valhalla.component;

import cn.fw.shirasawa.sdk.mq.FollowApproveDTO;
import cn.fw.valhalla.domain.db.follow.ClueTask;
import cn.fw.valhalla.domain.db.follow.FollowClue;
import cn.fw.valhalla.domain.enums.ClueStatusEnum;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.enums.TaskStateEnum;
import cn.fw.valhalla.service.bus.cust.CustomerBizService;
import cn.fw.valhalla.service.data.ClueTaskService;
import cn.fw.valhalla.service.data.FollowClueService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Objects;

/**
 * @author : kurisu
 * Date: 2020/9/16
 * Time: 10:57
 * Description:
 */
@Slf4j
@Component
@RocketMQMessageListener(topic = FollowApproveDTO.FOLLOW_APPROVE_TOPIC, consumerGroup = "${spring.application.name}_follow_flow_result")
public class FollowFlowConsumer implements RocketMQListener<FollowApproveDTO> {

    private final CustomerBizService customerBizService;
    private final FollowClueService followClueService;
    private final ClueTaskService clueTaskService;


    @Autowired
    public FollowFlowConsumer(final CustomerBizService customerBizService,
                              final FollowClueService followClueService,
                              final ClueTaskService clueTaskService) {
        this.customerBizService = customerBizService;
        this.followClueService = followClueService;
        this.clueTaskService = clueTaskService;
    }

    @Override
    public void onMessage(FollowApproveDTO dto) {
        log.info("跟进审批通过MQ消息回调：{}", JSONObject.toJSONString(dto));
        try {
            FollowClue clue = null;
            if (FollowTypeEnum.AC.getValue().equals(dto.getType())) {
                clue = followClueService.getOne(Wrappers.<FollowClue>lambdaQuery()
                        .eq(FollowClue::getPlateNo, dto.getPlateNo())
                        .eq(FollowClue::getClueState, ClueStatusEnum.ONGOING)
                        .eq(FollowClue::getClueType, dto.getType())
                );
            } else {
                clue = followClueService.getOne(Wrappers.<FollowClue>lambdaQuery()
                        .eq(FollowClue::getVin, dto.getVin())
                        .eq(FollowClue::getClueState, ClueStatusEnum.ONGOING)
                        .eq(FollowClue::getClueType, dto.getType())
                );
            }

            if (Objects.isNull(clue)) {
                return;
            }

            doSomething(clue);
        } catch (Exception e) {
            log.error("跟进审批通过处理异常", e);
            throw e;
        }
    }

    public void doSomething(FollowClue clue) {
        ClueTask task = clueTaskService.getOne(Wrappers.<ClueTask>lambdaQuery()
                        .eq(ClueTask::getType, clue.getClueType())
                        .eq(ClueTask::getClueId, clue.getId())
                        .eq(ClueTask::getState, TaskStateEnum.ONGOING)
                , Boolean.FALSE
        );
        if (Objects.isNull(task)) {
            return;
        }
        customerBizService.abandon(task);
    }
}
